############################# PSQF 6272 Example 5 using R #####################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("Hmisc")) install.packages("Hmisc")
library(Hmisc) # To get bivariate correlations with p-values
# Note: has routine "describe" that competes with psych package

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lme4")) install.packages("lme4")
library(lme4) # To estimate MLMs using lmer

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC using lmer

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

# Clear workspace (re-run as needed for troubleshooting purposes)
#rm(list = ls())

################################################################################
#####          BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 5             #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/23_PSQF6272/PSQF6272_Example5/"
filename = "Example5_Data.xlsx"
setwd(dir=filesave)

# Load Jonathan's custom R functions from folder within working directory
functions = paste0("R functions/",dir("R functions/"))
temp = lapply(X=functions, FUN=source)

#  Import trimmed example excel data file from sheet "Hox"
Example5 = read_excel(paste0(filesave,filename), sheet="Hox") 
# Convert to data frame to use in analysis
Example5 = as.data.frame(Example5)

# Labels for original variables in comments only
#childID=    "childID: Child ID"
#PschoolID=  "PschoolID: Primary School ID"
#SschoolID=  "SschoolID: Secondary School ID"
#achieve=    "achieve: Child Achievement Outcome"
#ses=        "ses: Child Socio-Economic Status"
#Pdenom=     "Pdenom: Primary School Denomination"
#Sdenom=     "Sdenom: Secondary School Denomination"

# Open external file to save results to (turn off to use console instead)
txtStart(file=paste0(filesave,"Example5_R_Output.txt"))

print("R Descriptive Statistics for Child Variables")
describe(x=Example5[ , c("achieve","ses")])


# Get means per primary school for child variables using Jonathan's function
Example5 = addUnitMeans(data=Example5, unitVariable="PschoolID",
                        meanVariables=c("achieve","ses"), newNames=c("PMachieve","PMses"))

print("R Descriptive Statistics and Correlations for Primary Schools")
Primary = unique(Example5[,c("PschoolID","PMachieve","PMses","NperPschoolID")])
describe(x=Primary[ , c("PMachieve","PMses","NperPschoolID")])
rcorr(x=as.matrix(Primary[ , c("PMachieve","PMses","NperPschoolID")]), type="pearson")


# Get means per secondary school for child variables using Jonathan's function
Example5 = addUnitMeans(data=Example5, unitVariable="SschoolID",
                        meanVariables=c("achieve","ses"), newNames=c("SMachieve","SMses"))

print("R Descriptive Statistics and Correlations for Secondary Schools")
Secondary = unique(Example5[,c("SschoolID","SMachieve","SMses","NperSschoolID")])
describe(x=Secondary[ , c("SMachieve","SMses","NperSschoolID")])
rcorr(x=as.matrix(Secondary[ , c("SMachieve","SMses","NperSschoolID")]), type="pearson")


# Create unique ID variable for primary/secondary school combination
uniqueIDs = unique(Example5[c("PschoolID", "SschoolID")])
uniqueIDs$UniqueID = 100000 + 1:nrow(uniqueIDs)
# Merge unique ID back into child-level data
Example5 = merge(x=Example5, y=uniqueIDs, by=c("PschoolID","SschoolID"))

# Get means per unique combination of primary/secondary school for child variables using Jonathan's function
Example5 = addUnitMeans(data=Example5, unitVariable=c("UniqueID"),
                        meanVariables=c("achieve","ses"), newNames=c("PSMachieve","PSMses"))

print("R Descriptive Statistics and Correlations for Unique Primary/Secondary Combination")
Unique = unique(Example5[,c("UniqueID","PSMachieve","PSMses","NperUniqueID")])
table(x=Unique$NperUniqueID, useNA="ifany")
prop.table(table(x=Unique$NperUniqueID, useNA="ifany"))
describe(x=Unique[ , c("PSMachieve","PSMses","NperUniqueID")])
rcorr(x=as.matrix(Unique[ , c("PSMachieve","PSMses","NperUniqueID")]), type="pearson")


# Constant-centered predictors
Example5$ses4    = Example5$ses - 4
Example5$PMses4  = Example5$PMses - 4
Example5$SMses4  = Example5$SMses - 4
Example5$PSMses4 = Example5$PSMses - 4

# Cluster-mean-centered level-1 child predictors
Example5$WPses  = Example5$ses - Example5$PMses
Example5$WSses  = Example5$ses - Example5$SMses
Example5$WPSses = Example5$ses - Example5$PSMses
# WPses=  "WPses: Within-Primary Centered Child SES"
# WSses=  "WSses: Within-Secondary Centered Child SES"
# WPSses= "WPSses: Within-Primary/Secondary Centered Child SES"

print("R Descriptives and Correlations in Child-Level Data")
var(Example5$WPSses)
rcorr(x=as.matrix(Example5[ , c("PMses","SMses","PSMses","WPses","WSses","WPSses")]), type="pearson")


################################################################################
#####                 BEGIN EMPTY MODELS FOR EXAMPLE 5                     #####
################################################################################

print("R Model 1a: Empty Means, Primary Random Intercept")
Model1a = lmer(data=Example5, REML=TRUE, formula=achieve~1+(1|PschoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model1a, chkREML=FALSE); summary(Model1a, ddf="Satterthwaite")
print("Show intraclass correlation and its LRT")
icc(Model1a); ranova(Model1a)

txtComment(" ") # insert blank space in output


print("R Model 1b: Empty Means, Primary by Secondary Crossed Random Intercepts")
Model1b = lmer(data=Example5, REML=TRUE, formula=achieve~1+(1|PschoolID)+(1|SschoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model1b, chkREML=FALSE); summary(Model1b, ddf="Satterthwaite")
print("Show LRT for each random intercept"); ranova(Model1b)

print("Show saved variances from model 1b")
as.data.frame(VarCorr(Model1b))

# Compute total variance from model 1b saved variances
total1b = as.data.frame(VarCorr(Model1b))[1,4] +
          as.data.frame(VarCorr(Model1b))[2,4] +
          as.data.frame(VarCorr(Model1b))[3,4]

print("ICC for Children in Same Primary School but Different Secondary Schools")
as.data.frame(VarCorr(Model1b))[1,4] / total1b

print("ICC for Children in Same Secondary School but Different Primary Schools")
as.data.frame(VarCorr(Model1b))[2,4] / total1b

print("ICC for Children in Same Primary and Secondary Schools")
(as.data.frame(VarCorr(Model1b))[1,4] +
 as.data.frame(VarCorr(Model1b))[2,4]) / total1b

print("Show ICC for each school type"); icc(Model1b, by_group=TRUE)
print("Show ICC for same primary and secondary"); icc(Model1b)

txtComment(" ") # insert blank space in output


print("R Model 1c: Empty Means, Primary by Secondary and Unique Crossed Random Intercepts")
Model1c = lmer(data=Example5, REML=TRUE, formula=achieve~1+(1|PschoolID)+(1|SschoolID)+(1|UniqueID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model1c, chkREML=FALSE); summary(Model1c, ddf="Satterthwaite")
print("Show LRT for each random intercept"); ranova(Model1c)

print("Show saved variances from model 1c")
as.data.frame(VarCorr(Model1c))

# Compute total variance from model 1c object
total1c = as.data.frame(VarCorr(Model1c))[1,4] +
          as.data.frame(VarCorr(Model1c))[2,4] +
          as.data.frame(VarCorr(Model1c))[3,4] +
          as.data.frame(VarCorr(Model1c))[4,4] 

print("ICC for Children in Same Primary but Different Secondary Schools")
as.data.frame(VarCorr(Model1c))[2,4] / total1c

print("ICC for Children in Same Secondary but Different Primary Schools")
as.data.frame(VarCorr(Model1c))[3,4] / total1c

print("ICC for Children Within Same Primary and Same Secondary Schools")
(as.data.frame(VarCorr(Model1c))[1,4] +
 as.data.frame(VarCorr(Model1c))[2,4] + 
 as.data.frame(VarCorr(Model1c))[3,4]) / total1c 

print("Show ICC for each school type"); icc(Model1c, by_group=TRUE)
print("Show ICC for same primary and secondary"); icc(Model1c)

txtComment(" ") # insert blank space in output


print("R Empty Means, Three-Way Crossed Model for SES Predictor")
EmptySES = lmer(data=Example5, REML=TRUE, formula=ses~1+(1|PschoolID)+(1|SschoolID)+(1|UniqueID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(EmptySES, chkREML=FALSE); summary(EmptySES, ddf="Satterthwaite")
print("Show intraclass correlation by random type and its LRT")
icc(EmptySES, by_group=TRUE); icc(EmptySES); ranova(EmptySES)

txtComment(" ") # insert blank space in output


################################################################################
#####             BEGIN CONDITIONAL MODELS FOR EXAMPLE 5                   #####
################################################################################

print("R Model 2: Model 1b + Primary*Secondary School Denomination")
Model2 = lmer(data=Example5, REML=TRUE, 
              formula=achieve~1+Pdenom+Sdenom+Pdenom:Sdenom+(1|PschoolID)+(1|SschoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2, chkREML=FALSE); summary(Model2, ddf="Satterthwaite")

txtComment(" ") # insert blank space in output


print("R Model 3a: Add Child SES")
Model3a = lmer(data=Example5, REML=TRUE, formula=achieve~1+Pdenom+Sdenom+Pdenom:Sdenom
                +ses4+(1|PschoolID)+(1|SschoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model3a, chkREML=FALSE); summary(Model3a, ddf="Satterthwaite")

txtComment(" ") # insert blank space in output


print("R Model 3b: Add Child SES Contextual Effects")
Model3b = lmer(data=Example5, REML=TRUE, formula=achieve~1+Pdenom+Sdenom+Pdenom:Sdenom
                +ses4+PMses4+SMses4+PSMses4+(1|PschoolID)+(1|SschoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model3b, chkREML=FALSE); summary(Model3b, ddf="Satterthwaite")

txtComment(" ") # insert blank space in output


print("R Model 4a: Add Random WPS-Centered Child SES across Primary Schools")
Model4a = lmer(data=Example5, REML=TRUE, formula=achieve~1+Pdenom+Sdenom+Pdenom:Sdenom
               +ses4+PMses4+SMses4+PSMses4+(1+WPSses|PschoolID)+(1|SschoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model4a, chkREML=FALSE); summary(Model4a, ddf="Satterthwaite")
print("LRT for random slope"); ranova(Model4a)

txtComment(" ") # insert blank space in output


print("R Model 4b: Model 3b + Random WPS-Centered Child SES across Secondary Schools")
Model4b = lmer(data=Example5, REML=TRUE, formula=achieve~1+Pdenom+Sdenom+Pdenom:Sdenom
               +ses4+PMses4+SMses4+PSMses4+(1|PschoolID)+(1+WPSses|SschoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model4a, chkREML=FALSE); summary(Model4b, ddf="Satterthwaite")
print("LRT for random slope"); ranova(Model4b)

txtComment(" ") # insert blank space in output


#######################################################################################
# Close external file of saved results
txtStop()



