// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

///////////////////////////////////////////////////////////////////////////////////////
////////               BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 5          ///////
////////                  CHANGE "filesave" TO YOUR FOLDER PATH                 ///////
///////////////////////////////////////////////////////////////////////////////////////

// Define global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
   global filesave "C:\Dropbox\23_PSQF6272\PSQF6272_Example5"

// Open trimmed example excel data file from sheet "grade10" and clear away existing data
   clear // clear memory in case of open data
   import excel "$filesave\Example5_Data.xlsx", firstrow case(preserve) sheet("Hox") clear 
   
// Add labels to original variables
   label variable childID    "childID: Child ID"
   label variable PschoolID  "PschoolID: Primary School ID"
   label variable SschoolID  "SschoolID: Secondary School ID"
   label variable achieve    "achieve: Child Achievement Outcome"
   label variable ses        "ses: Child Socio-Economic Status"
   label variable Pdenom     "Pdenom: Primary School Denomination"
   label variable Sdenom     "Sdenom: Secondary School Denomination"
     
// Open external file to save results to
   log using $filesave\Example5_STATA_Output.log, replace   
     
display "STATA Descriptives for Child Variables"
summarize achieve ses
     
// Get means per primary school for child variables
   sort PschoolID
   egen PrimN =     count(achieve), by(PschoolID)
   egen PMachieve = mean(achieve),  by(PschoolID)
   egen PMses =     mean(ses),      by(PschoolID)
   label variable PMachieve "PMachieve: Primary Mean Child Achievement"
   label variable PMses     "PMses: Primary Mean Child SES"

display "STATA Descriptives and Correlations for Primary Schools"
preserve  // Save for later use, then compute primary school dataset
collapse  PMachieve Pdenom PMses PrimN, by(PschoolID)
format    PMachieve Pdenom PMses PrimN  %4.2f
summarize PMachieve Pdenom PMses PrimN, format
pwcorr    PMachieve Pdenom PMses PrimN, sig
restore   // Go back to child-level dataset     
     
// Get means per secondary school for child variables
   sort SschoolID
   egen SecN =      count(achieve), by(SschoolID)
   egen SMachieve = mean(achieve),  by(SschoolID)
   egen SMses =     mean(ses),      by(SschoolID)
   label variable SMachieve "SMachieve: Secondary Mean Child Achievement"
   label variable SMses     "SMses: Secondary Mean Child SES"

display "STATA Descriptives and Correlations for Secondary Schools"
preserve  // Save for later use, then compute secondary school dataset
collapse  SMachieve Sdenom SMses SecN, by(SschoolID)
format    SMachieve Sdenom SMses SecN  %4.2f
summarize SMachieve Sdenom SMses SecN, format
pwcorr    SMachieve Sdenom SMses SecN, sig
restore   // Go back to child-level dataset     

// Get means per unique combination of primary/secondary school for child variables
   sort PschoolID SschoolID
   egen UniqueID =   group(PschoolID SschoolID)  // Create unique ID
   egen UniqueN =    count(achieve), by(PschoolID SschoolID)
   egen PSMachieve = mean(achieve),  by(PschoolID SschoolID)
   egen PSMses =     mean(ses),      by(PschoolID SschoolID)
   label variable PSMachieve "PSMachieve: Unique Primary/Secondary Mean Child Achievement"
   label variable PSMses     "PSMses: Unique Primary/Secondary Mean Child SES"  
     
display "STATA Descriptives and Correlations for Unique Primary/Secondary Combination"
preserve  // Save for later use, then compute secondary school dataset
collapse  PSMachieve PSMses UniqueN, by(PschoolID SschoolID)
format    PSMachieve PSMses UniqueN  %4.2f
tabulate  UniqueN
summarize PSMachieve PSMses UniqueN, format
pwcorr    PSMachieve PSMses UniqueN, sig
restore   // Go back to child-level dataset     
     
// Constant-centered predictors
   gen ses4    = ses - 4 
   gen PMses4  = PMses - 4  
   gen SMses4  = SMses - 4  
     gen PSMses4 = PSMses - 4
// Cluster-mean-centered level-1 child predictors
   gen WPses  = ses - PMses   // Within primary only
   gen WSses  = ses - SMses   // Within secondary only
   gen WPSses = ses - PSMses  // Within unique combination
   label variable WPses  "WPses: Within-Primary Centered Child SES"
   label variable WSses  "WSses: Within-Secondary Centered Child SES"
   label variable WPSses "WPSses: Within-Primary/Secondary Centered Child SES"

display "STATA Descriptives and Correlations in Child-Level Data"
summarize WPSses, detail
pwcorr PMses SMses PSMses WPses WSses WPSses, sig


////////////////////////////////////////////////////////////////////////////////
////////                 BEGIN EMPTY MODELS FOR EXAMPLE 5                ///////
////////////////////////////////////////////////////////////////////////////////

display "STATA Model 1a: Empty Means, Primary Random Intercept Only"
mixed achieve  , || PschoolID: ,    reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat icc                       // Intraclass correlation 
estimates store FitEmpty1       // Save for LRT

display "STATA Model 1b: Empty Means, Primary by Secondary Crossed Random Intercepts"
mixed achieve  , || _all: R.SschoolID || PschoolID: ,  ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estimates store FitEmpty2       // Save for LRT
lrtest FitEmpty2 FitEmpty1      // LRT for secondary intercept variance

display "STATA Model 1c: Empty Means, Primary by Secondary and Unique Crossed Random Intercepts"
mixed achieve  , || _all: R.SschoolID || PschoolID: || UniqueID: ,  ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estimates store FitEmpty3       // Save for LRT
lrtest FitEmpty3 FitEmpty2      // LRT for unique intercept variance

display "STATA Empty Means, Three-Way Crossed Model for SES Predictor"
mixed ses  , || _all: R.SschoolID || PschoolID: || UniqueID: ,  ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog

     
////////////////////////////////////////////////////////////////////////////////
////////             BEGIN CONDITIONAL MODELS FOR EXAMPLE 5              ///////
////////////////////////////////////////////////////////////////////////////////

display "STATA Model 2: Model 1b + Primary*Secondary School Denomination"
mixed achieve c.Pdenom c.Sdenom c.Pdenom#c.Sdenom ,  ///
      || _all: R.SschoolID || PschoolID: ,           ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  

display "STATA Model 3a: Add Child SES"
mixed achieve c.Pdenom c.Sdenom c.Pdenom#c.Sdenom c.ses4, ///
      || _all: R.SschoolID || PschoolID: ,                ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  

display "STATA Model 3b: Add Child SES Contextual Effects"
mixed achieve c.Pdenom c.Sdenom c.Pdenom#c.Sdenom  ///
              c.ses4 c.PMses4 c.SMses4 c.PSMses4,  ///
      || _all: R.SschoolID || PschoolID: ,         ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estimates store FitFix          // Save for LRT

display "STATA Model 4a: Add Random WPS-Centered Child SES across Primary Schools"
mixed achieve c.Pdenom c.Sdenom c.Pdenom#c.Sdenom ///
              c.ses4 c.PMses4 c.SMses4 c.PSMses4, ///
      || _all: R.SschoolID || PschoolID: WPSses, cov(un) ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat recovariance, relevel(PschoolID) correlation  // Random effect correlations
estimates store FitRandP        // Save for LRT
lrtest FitRandP FitFix          // LRT for random slope over primary?

display "STATA Model 4b: Model 3b + Random WPS-Centered Child SES across Secondary Schools"
mixed achieve c.Pdenom c.Sdenom c.Pdenom#c.Sdenom  ///
              c.ses4 c.PMses4 c.SMses4 c.PSMses4,  ///
      || _all: R.PschoolID || SschoolID: WPSses, cov(un) ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat recovariance, relevel(SschoolID) correlation  // Random effect correlations
estimates store FitRandS        // Save for LRT
lrtest FitRandS FitFix          // LRT for random slope over secondary?
 

// Close external file of saved results
   log close  

